@@ -186,7 +186,7 @@ var ThreeModel = widgets.WidgetModel.extend({
 
         // Handle changes in data widgets/union properties
         this.datawidget_properties.forEach(function(propName) {
-            dataserializers.listenToUnion(this, propName, this.onChildChanged.bind(this), false);
+            dataserializers.listenToUnion(this, propName, this.onDataChanged.bind(this), false);
         }, this);
 
         this.on('change', this.onChange, this);
@@ -349,6 +349,16 @@ var ThreeModel = widgets.WidgetModel.extend({
         this.trigger('childchange', this);
     },
 
+    onDataChanged: function(model, options) {
+        console.debug('child data changed: ' + model.model_id);
+        // Treat a data widget change as if data attribute changed
+        // Note: hasChanged() etc won't identify the attribute, so
+        // property_mappers should be used for union properties.
+        this.onChange(model, options);
+        // Propagate up hierarchy:
+        this.trigger('childchange', this);
+    },
+
     // push data from model to three object
     syncToThreeObj: function(force) {
 
@@ -624,11 +634,13 @@ var ThreeModel = widgets.WidgetModel.extend({
             f.a,
             f.b,
             f.c,
-            f.normal.toArray(),
-            this.convertColorThreeToModel(f.color),
+            f.vertexNormals.length > 0
+                ? this.convertVectorArrayThreeToModel(f.vertexNormals)
+                : f.normal.toArray(),
+            f.vertexColors.length > 0
+                ? this.convertColorArrayThreeToModel(f.vertexColors)
+                : this.convertColorThreeToModel(f.color),
             f.materialIndex,
-            this.convertVectorArrayThreeToModel(f.vertexNormals),
-            this.convertColorArrayThreeToModel(f.vertexColors),
         ];
     },
 
